#!/bin/bash

function print()
{
    # prints one comment line followed by any number of "key = value" lines
    echo "# $1 : $2 (default=$3)"
    shift; shift; shift;
    while (($# > 0)) ; do
	echo "$1 = $2"
	shift; shift;
    done
    echo
}

################################### Planet #######################################
 
function radius { print 'Planetary radius' real 37122E6 radius $1 
}
function g      { print Gravity real 9.80616 g $1 
}
function omega { print 'Planetary rotation rate' real 7.292E-5 omega $1 
}
function kappa { print 'kappa=Rd/cpp' real 0.2857143 kappa $1 
}
function cpp	 { print 'Heat capacity at constant pressure' real 1004.70885 cpp $1
}
function preff { print 'Reference pressure' real 101325 preff $1 
}
function scale_factor { print 'Scale factor for small planet experiment' real 1. scale_factor $1 
}

##################################### Mesh #####################################

function nbp   { print 'Number of subdivisions on a main triangle' integer 40 nbp $1 
}
function split { print 'Sub splitting of main rhombus' integer 1 nsplit_i $1 nsplit_j $2 
}
function llm   { print 'Number of vertical layers' integer 19 llm $1 
}
function disvert { print 'Vertical grid' '[std|ncar|ncar30l]' std disvert $1 
}
function ncar_dz { print 'Model layer thickness in m, used if disvert=ncar' real 300 ncar_dz $1
}
function optim_it { print 'Mesh optimisation : number of iterations' integer 0 optim_it $1 
}

################################# Numerics ###################################

function scheme { print 'Time-stepping scheme' '[euler|leapfrog_matsuno|runge_kutta]' runge_kutta scheme $1 
}
function time_step { print 'Time step in s' real 480 dt $1 
}
function matsuno { print 'Matsuno period' integer 5 matsuno_period $1 
}
function itau_adv { print 'Advection called every itau_adv time steps' integer 2 itau_adv $1 
}
function nqtot { print 'Number of tracers' integer 1 nqtot $1 
}
function caldyn { print 'Equations solved' '[gcm|adv]' gcm caldyn $1 
}
function caldyn_conserv { print 'Discretization of Coriolis term' '[energy|enstophy]' enstrophy caldyn_conserv $1 
}
function caldyn_eta { print 'Vertical coordinate' '[eta_mass|eta_lag]' mass caldyn_eta $1 
}
function boussinesq { print 'Boussinesq approximation, requires eta_lag' '[.FALSE.|.TRUE.]' .FALSE. boussinesq $1
}
############################### Time and output ###############################

function time_style() { print 'Time style' '[none|lmd]' lmd time_style $1 
}
function ndays() { print 'Number of days' integer ?? ndays $1 
}
function run_length() { print 'Run length in s' real ?? run_length $1 
}
function day_step { print 'Number of time steps per day' integer ?? day_step $1 
}
function write_period { print 'Interval in s between two outputs' integer ?? write_period $1 
}
function out_pressure_level { print 'Pressure level to which interpolate T' real 0 out_pression_level $1
}

################################ Physics ######################################

function etat0 { 
    OPTS=$(printf '\n#   [jablonowsky06|academic|dcmip[1-4]|heldsz|dcmip2_schaer_noshear|dcmip3]')
    print 'Initial state' "$OPTS" jablonowsky06 etat0 $1 
}
function dissip
{
    print 'Dissipation time for grad(div)' real 5000 tau_graddiv $1
    print 'Exponent of grad(div) disspation' integer 1 nitergdiv $2
    print 'Dissipation time for curl(curl)' real 5000 tau_gradrot $3
    print 'Exponent of curl(curl) disspation' integer 1 nitergrot $2
    print 'Dissipation time for div(grad)' real 5000 tau_divgrad $3
    print 'Exponent of div(grad) disspation' integer 1 niterdivgrad $2
}
function guided_type { print 'Type of guiding terms' '[none|dcmip1]' none guided_type $1 
}

function physics { print 'Physics package' '[automatic|dcmip]' automatic physics $1
}

#################################### DCMIP #################################
function dcmip1_def()
{
    OPTS=$(printf '\n#    [const|slotted_cyl|cos_bell|dbl_cos_bell_q1|dbl_cos_bell_q2|complement|hadley|dcmip11]')
    print 'DCMIP 1.1 advection test, initial tracer field' "$OPTS" cos_bell dcmip1_shape $1
    print 'DCMIP 1.1 advection test, wind field' '[solid|deform|hadley]' deform dcmip1_shape $2
}
function dcmip_refs()
{
    print 'Reference temperature T0 for DCMIP test cases' real 300 ncar_T0 $1
    print 'Reference pressure p0 for DCMIP test cases, used by disvert if disver=ncar' real 1e5 ncar_p0 $2
    print 'Exponent for B(eta), used by disvert if disvert=ncar' integer 1 ncar_disvert_c $3
}
function dcmip_physics()
{
    physics dcmip
    print 'DCMIP physics' '[0|1]' 1 dcmip_physics $1
}
function dcmip4_testcase { print 'DCMIP 4 Test case' '[1|2]' 1 dcmip4_testcase $1 
}

####################### Functions generating parameter files ####################

function params {
    # executes any number of "command arg" commands
    while (($# > 0)) ; do
	$1 $2
	shift; shift;
    done
}

function group { 
    # prints one separation line then executes any number of "command arg" commands
    echo
    echo "#---------------- $1 ----------------"
    echo
    shift
    params $*
}

function williamson91()
{
    group 'Mesh' nbp 40 llm 1 optim_it 100 ; split 2 2
    group 'Numerics' caldyn gcm scheme runge_kutta time_step 480 caldyn_conserv energy
    group 'Time and output' out_pressure_level 85000 write_period 21600 run_length 864000
    group 'Physical parameters' radius 6.37122e6 g 9.80616 caldyn_eta eta_lag boussinesq .TRUE. etat0 williamson91.6
    dissip 5000 2 5000 2 5000 2    
}

function held_suarez()
{
    group 'Mesh' nbp 40 llm 19 disvert std optim_it 1000 ; split 2 2
    group 'Numerics' scheme runge_kutta itau_adv 1 day_step 180 caldyn_conserv energy
    group 'Time' time_style lmd ndays 1200 write_period 86400
    group 'Planet' scale_factor 1 radius 6.371e6 g 9.8 omega 7.292e-5 kappa 0.2857143 cpp 1004 preff 1e5
    group 'Physical parameters' etat0 held_suarez nqtot 1
    dissip 50000 2 50000 2 50000 2
}

function const_dcmip
{
    group 'Planet' radius 6.37122e6 g 9.80616 kappa 0.2857143 cpp 1004.5 preff 1e5 omega $1 
    group 'Constants' ; dcmip_refs 300 1e5 1
}

function dcmip11
{
    echo 'INCLUDEDEF = const.def'
    group 'Mesh' nbp 80 llm 60 disvert ncar ncar_dz 200 optim_it 100 ; split 2 2
    group 'Numerics' caldyn adv time_step 180 caldyn_conserv energy
    group 'Time' run_length 1036800 write_period 86400
    group 'Physical parameters' guided_type dcmip1 ; dcmip1_def dcmip11 deform
}

function dcmip21
{
    echo 'INCLUDEDEF = const.def'
    group 'Mesh' nbp 64 llm 60 disvert ncar ncar_dz 500 optim_it 100 ; split 2 2
    group 'Numerics' time_step 300 caldyn_conserv energy
    dissip 1800 1 1800 2 1800 2
    group 'Time' run_length 3.6e6 write_period 50000
    group 'Physical parameters' scale_factor 500 guided_type dcmip1 nqtot 1 etat0 dcmip2_schaer_noshear
}

function dcmip3
{
    const_dcmip 0
    group 'Mesh' nbp 80 llm 10 disvert ncar ncar_dz 1000 optim_it 100 ; split 2 2
    group 'Numerics' caldyn gcm scheme runge_kutta time_step 240
    dissip 1800 1 1800 2 1800 2
    group 'Time' run_length 4.5e5 write_period 12500
    group 'Physical parameters' omega 0 scale_factor 125 nqtot 1 etat0 dcmip3
}

function dcmip42
{
    echo 'INCLUDEDEF = const.def'
    group 'Mesh' nbp 80 llm 30 disvert ncar30l optim_it 100 ; split 2 2
    group 'Numerics' caldyn gcm scheme runge_kutta time_step 240
    dissip 1800 1 1800 2 1800 2
    group 'Time' run_length 3.6e6 write_period 21600
    group 'Physical parameters' scale_factor 1 nqtot 2 etat0 dcmip4 dcmip4_testcase 2 dcmip_physics 1
}

function dcmip5
{
    echo 'INCLUDEDEF = const.def'
    group 'Mesh' nbp 64 llm 30 disvert ncar30l optim_it 1000 ; split 2 2
    group 'Numerics' caldyn gcm scheme runge_kutta time_step 300
    dissip 10800 1 10800 2 10800 2
    group 'Time' run_length 864000 write_period 7200
    group 'Physical parameters' scale_factor 1 nqtot 1 etat0 dcmip5 dcmip_physics 0
}
    
########## Main program : generate all *.def files in param_sets ##############

function mkcd { cd $ROOT ; mkdir -p $1 ; cd $1
}

set -o errexit
ROOT=$PWD

mkcd climate/Held_Suarez
held_suarez > run.def

mkcd shallow_water/williamson91
williamson91 > run6.def

mkcd dcmip2012
const_dcmip 7.292e-5 > const.def
dcmip11 > run11.def
dcmip21 > run21.def
dcmip3 > run3.def
dcmip42 > run42.def
dcmip5 > run5.def
